program usr2cma;

{This program is copyright 1995 by Bob Dunnell.  You are granted the
right to compile and use it for your own purposes.  You may also use
it in other programs as long as I am given credit.}

{This program outputs a TriBBS 5.1 user data file to a comma-delimited
text file.  I have renamed the TriBBS 5.1 API fields to reduce code
size.  The TriBBS 5.1 API should be consulted for any sort of
questions about what each individual field is for.}

{The program reads records from users.dat, parses them to be
comma-delimited, and writes them out to a comma-delimited text
file (users.txt).  It is especially useful for writing conversion
programs to other BBS softwares or for importing the BBS user
files into a database program.}

{The program will convert all commas in strings to the tilde (~)
character.  This is so a comma in a string does not interfere with
the comma-delimited record.}

uses crt, dos;
type
     USERSDAT = record

     {This is a modified version of the TriBBS 5.1 API data structure
      for user information.  It is "Pascalized" and some of the names
      have been shortened.  Please refer to the TriBBS API for more
      in-depth discussion of this structure}

          Name : array [1..31] of char;
          Password :array [1..16] of char;
          city :array [1..31] of char ;
          phone :array [1..13] of char;
          Birthday :array [1..9] of char;
          firstcall :array [1..9] of char;
          lastcall :array [1..15] of char;
          lastfile :array [1..9] of char;
          seclevel,
          callnum,
          timeleft,
          messconf,
          filearea,
          protocol,
          expert,
          lockout,
          deleted,
          callstoday,
          inclogon1qwk,
          incgoodbyeqwk,
          incbulletqwk,
          incfilesqwk,
          incnewssqwk,
          qwknet : integer;
          uploads,
          downloads,
          ksent,
          kdown,
          messages : longint;
          editor,
          chatav,
          chknwmail : integer;
          subexp : array [1..9] of char;
          Alias : array [1..31] of char;
          downtoday : integer;
          bytestoday : longint;
          resdata : array [1..19] of char;
     end;

     USERSDATFILE = file of usersdat;

var
   inf   :  usersdatfile;   {users.dat}
   outf  :  text;           {users.txt}
   ts    :  string;         {temporary string}
   cuser :  usersdat;       {current user}
   i     :  integer;        {index for online counter}

   function FileExists(filename : string) : boolean;

   {check to see if a file exists}

   var
      r: file;
   begin
        {$I-}
        Assign(r, FileName);
        Reset(r);
        Close(r);
        {$I+}
        FileExists := (IOResult = 0) and
        (FileName <> '');
   end;

   procedure parseuser(usr : usersdat; var s : string);

       function istr(i: longint): string;

       {returns a string for a passed integer}

       var
          s: string[11];
       begin
          str(i, s);
          istr := s;
       end;  {parseuser.istr}

       function packit(ps:string):string;

       {packs a string, removing all trailing spaces and nulls}

       var j,k : integer;
       begin
        j:=length(ps);
        for k:=1 to j do
            if ps[k] = ',' then ps[k] := '~';
        for k:=j downto 1 do
        begin
             if (ps[k] <> #32) and (ps[k] <> #0) then
             begin
                  packit:=copy(ps,1,k);
                  exit;
                  end;
        end;
       end;  {parseuser.packit}

   {parses the user record into one single comma-delimited string}

   const c = ',';
   var
      ds : array [1..$F] of string;

   begin

     with usr do
     begin

          {first, convert segments of the record to strings}

          ds[1] := packit(name)+c+packit(password)+c+packit(city)+c+packit(phone)+c+packit(birthday)+c;
          ds[2] := packit(firstcall)+c+packit(lastcall)+c+packit(lastfile)+c;
          ds[3] := istr(seclevel)+c+istr(callnum)+c+istr(timeleft)+c;
          ds[4] := istr(messconf)+c+istr(filearea)+c+istr(protocol)+c;
          ds[5] := istr(expert)+c+istr(lockout)+c+istr(deleted)+c;
          ds[6] := istr(callstoday)+c+istr(inclogon1qwk)+c;
          ds[7] := istr(incgoodbyeqwk)+c+istr(incbulletqwk)+c;
          ds[8] := istr(incfilesqwk)+c+istr(incnewssqwk)+c;
          ds[9] := istr(qwknet)+c+istr(uploads)+c;
          ds[$A] := istr(downloads)+c+istr(ksent)+c;
          ds[$B] := istr(kdown)+c+istr(messages)+c;
          ds[$C] := istr(editor)+c+istr(chatav)+c;
          ds[$D] := istr(chknwmail)+c+packit(subexp)+c;
          ds[$E] := packit(Alias)+c+istr(downtoday)+c;
          ds[$F] := istr(bytestoday);

          {note that the reserved data field is omitted since it does
          not contain any data.  This can be modified to use it very
          easily, though}

          {then, add all the string segments up}

          s := concat(ds[1],ds[2],ds[3],ds[4],ds[5],ds[6],ds[7],ds[8],ds[9],
                      ds[$A],ds[$B],ds[$C],ds[$D],ds[$E],ds[$F]);

          end;
   end;  {parseuser}

begin  {main block}

     writeln('USR2CMA (C) 1995 by Bob Dunnell');
     writeln;
     writeln('Opening users.dat');

     {attempt to open users.dat}

     if not fileexists('users.dat') then
     begin
          writeln;
          writeln('Error -- users.dat not found.');
          writeln;
          writeln('This program must be run from node 1''s MWORK directory.');
          halt(1);
     end;

     assign(inf, 'users.dat');
     reset(inf);
     writeln;
     writeln('Creating output file users.txt.');
     writeln;

     {$I-}

     {attempt to open output file}

     assign(outf,'users.txt');
     rewrite(outf);
     if ioresult <> 0 then
     begin
          writeln('Error creating output file.');
          halt(1);
     end;

     write('Processing user record ');
     i := 0;
     while not eof(inf) do
     begin
          inc(i);
          gotoxy(24,wherey);
          write(i);

          read(inf, cuser);
          if ioresult <> 0 then
          begin
               writeln('Error reading users.dat');
               halt(1);
          end;

          parseuser(cuser, ts);

          writeln(outf, ts);
          if ioresult <> 0 then
          begin
               writeln('Error reading users.dat');
               halt(1);
          end;

     end;

     close(inf);
     flush(outf);
     close(outf);

     {$I+}

     writeln;
     writeln;
     if ioresult = 0 then
     begin
          writeln('Program completed succesfully.');
          writeln('Data is in users.txt.')
     end
     else
         writeln('There was an error -- users.txt may be incomplete.');

end.  {main block}

